@@ -1,9 +1,20 @@
 #ifndef EPIWORLD_MODELS_SEIRCONNECTED_HPP
 #define EPIWORLD_MODELS_SEIRCONNECTED_HPP
 
+/**
+ * @brief Template for a Susceptible-Exposed-Infected-Removed (SEIR) model with connected population
+ * 
+ * ![Model Diagram](../assets/img/seirconnected.png)
+ * 
+ * @ingroup connected_models
+ */
 template<typename TSeq = EPI_DEFAULT_TSEQ>
 class ModelSEIRCONN : public epiworld::Model<TSeq> 
 {
+private:
+    std::vector< epiworld::Agent<TSeq> * > infected;
+    void update_infected();
+
 public:
 
     static const int SUSCEPTIBLE = 0;
@@ -16,7 +27,7 @@ public:
 
     ModelSEIRCONN(
         ModelSEIRCONN<TSeq> & model,
-        std::string vname,
+        const std::string & vname,
         epiworld_fast_uint n,
         epiworld_double prevalence,
         epiworld_double contact_rate,
@@ -26,7 +37,7 @@ public:
     );
     
     ModelSEIRCONN(
-        std::string vname,
+        const std::string & vname,
         epiworld_fast_uint n,
         epiworld_double prevalence,
         epiworld_double contact_rate,
@@ -54,8 +65,45 @@ public:
         std::vector< int > queue_ = {}
     );
 
+    size_t get_n_infected() const { return infected.size(); }
+
+    /***
+     * @brief Compute expected generation time
+     * @param max_days Maximum number of days.
+     * @param max_contacts Maximum number of contacts.
+     */
+    std::vector< double > generation_time_expected(
+        int max_days = 200,
+        int max_contacts = 200
+    ) const;
+
 };
 
+template<typename TSeq>
+inline void ModelSEIRCONN<TSeq>::update_infected()
+{
+
+    infected.clear();
+    infected.reserve(this->size());
+
+    for (auto & p : this->get_agents())
+    {
+        if (p.get_state() == ModelSEIRCONN<TSeq>::INFECTED)
+        {
+            infected.push_back(&p);
+        }
+    }
+
+    Model<TSeq>::set_rand_binom(
+        this->get_n_infected(),
+        static_cast<double>(Model<TSeq>::par("Contact rate"))/
+            static_cast<double>(Model<TSeq>::size())
+    );
+
+    return;
+
+}
+
 template<typename TSeq>
 inline ModelSEIRCONN<TSeq> & ModelSEIRCONN<TSeq>::run(
     epiworld_fast_uint ndays,
@@ -74,13 +122,7 @@ inline void ModelSEIRCONN<TSeq>::reset()
 {
 
     Model<TSeq>::reset();
-
-    Model<TSeq>::set_rand_binom(
-        Model<TSeq>::size(),
-        static_cast<double>(
-            Model<TSeq>::par("Contact rate"))/
-            static_cast<double>(Model<TSeq>::size())
-        );
+    this->update_infected();
 
     return;
 
@@ -111,7 +153,7 @@ inline Model<TSeq> * ModelSEIRCONN<TSeq>::clone_ptr()
 template<typename TSeq>
 inline ModelSEIRCONN<TSeq>::ModelSEIRCONN(
     ModelSEIRCONN<TSeq> & model,
-    std::string vname,
+    const std::string & vname,
     epiworld_fast_uint n,
     epiworld_double prevalence,
     epiworld_double contact_rate,
@@ -133,13 +175,16 @@ inline ModelSEIRCONN<TSeq>::ModelSEIRCONN(
             if (ndraw == 0)
                 return;
 
+            ModelSEIRCONN<TSeq> * model = dynamic_cast<ModelSEIRCONN<TSeq> *>(m);
+            size_t ninfected = model->get_n_infected();
+
             // Drawing from the set
             int nviruses_tmp = 0;
             for (int i = 0; i < ndraw; ++i)
             {
                 // Now selecting who is transmitting the disease
                 int which = static_cast<int>(
-                    std::floor(m->size() * m->runif())
+                    std::floor(ninfected * m->runif())
                 );
 
                 /* There is a bug in which runif() returns 1.0. It is rare, but
@@ -149,35 +194,32 @@ inline ModelSEIRCONN<TSeq>::ModelSEIRCONN(
                  * https://gcc.gnu.org/bugzilla/show_bug.cgi?id=63176
                  * 
                  */
-                if (which == static_cast<int>(m->size()))
+                if (which == static_cast<int>(ninfected))
                     --which;
 
+                epiworld::Agent<TSeq> & neighbor = *model->infected[which];
+
                 // Can't sample itself
-                if (which == static_cast<int>(p->get_id()))
+                if (neighbor.get_id() == p->get_id())
                     continue;
 
-                // If the neighbor is infected, then proceed
-                auto & neighbor = m->get_agents()[which];
-                if (neighbor.get_state() == ModelSEIRCONN<TSeq>::INFECTED)
-                {
+                // The neighbor is infected by construction
+                auto & v = neighbor.get_virus();
 
-                    auto & v = neighbor.get_virus();
+                #ifdef EPI_DEBUG
+                if (nviruses_tmp >= static_cast<int>(m->array_virus_tmp.size()))
+                    throw std::logic_error("Trying to add an extra element to a temporal array outside of the range.");
+                #endif
+                    
+                /* And it is a function of susceptibility_reduction as well */ 
+                m->array_double_tmp[nviruses_tmp] =
+                    (1.0 - p->get_susceptibility_reduction(v, m)) * 
+                    v->get_prob_infecting(m) * 
+                    (1.0 - neighbor.get_transmission_reduction(v, m)) 
+                    ; 
+            
+                m->array_virus_tmp[nviruses_tmp++] = &(*v);
 
-                    #ifdef EPI_DEBUG
-                    if (nviruses_tmp >= static_cast<int>(m->array_virus_tmp.size()))
-                        throw std::logic_error("Trying to add an extra element to a temporal array outside of the range.");
-                    #endif
-                        
-                    /* And it is a function of susceptibility_reduction as well */ 
-                    m->array_double_tmp[nviruses_tmp] =
-                        (1.0 - p->get_susceptibility_reduction(v, m)) * 
-                        v->get_prob_infecting(m) * 
-                        (1.0 - neighbor.get_transmission_reduction(v, m)) 
-                        ; 
-                
-                    m->array_virus_tmp[nviruses_tmp++] = &(*v);
-
-                }
             }
 
             // No virus to compute
@@ -279,9 +321,25 @@ inline ModelSEIRCONN<TSeq>::ModelSEIRCONN(
     model.add_state("Infected", update_infected);
     model.add_state("Recovered");
 
+    // Adding update function
+    epiworld::GlobalFun<TSeq> update = [](
+        epiworld::Model<TSeq> * m
+        ) -> void
+        {
+
+            ModelSEIRCONN<TSeq> * model = dynamic_cast<ModelSEIRCONN<TSeq> *>(m);
+
+            model->update_infected();
+
+            return;
+
+        };
+
+    model.add_globalevent(update, "Update infected individuals");
+
 
     // Preparing the virus -------------------------------------------
-    epiworld::Virus<TSeq> virus(vname);
+    epiworld::Virus<TSeq> virus(vname, prevalence, true);
     virus.set_state(
         ModelSEIRCONN<TSeq>::EXPOSED,
         ModelSEIRCONN<TSeq>::RECOVERED,
@@ -292,7 +350,7 @@ inline ModelSEIRCONN<TSeq>::ModelSEIRCONN(
     virus.set_prob_recovery(&model("Prob. Recovery"));
     virus.set_incubation(&model("Avg. Incubation days"));
 
-    model.add_virus(virus, prevalence);
+    model.add_virus(virus);
 
     model.queuing_off(); // No queuing need
 
@@ -307,7 +365,7 @@ inline ModelSEIRCONN<TSeq>::ModelSEIRCONN(
 
 template<typename TSeq>
 inline ModelSEIRCONN<TSeq>::ModelSEIRCONN(
-    std::string vname,
+    const std::string & vname,
     epiworld_fast_uint n,
     epiworld_double prevalence,
     epiworld_double contact_rate,
@@ -347,4 +405,61 @@ inline ModelSEIRCONN<TSeq> & ModelSEIRCONN<TSeq>::initial_states(
 
 }
 
+template<typename TSeq>
+inline std::vector< double > ModelSEIRCONN<TSeq>::generation_time_expected(
+    int max_days,
+    int max_contacts
+) const
+{
+
+    // Retrieving total counts
+    std::vector< int > h_date;
+    std::vector< std::string > h_state;
+    std::vector< int > h_counts;
+    const auto this_const = dynamic_cast<const ModelSEIRCONN<TSeq> *>(this);
+    this_const->get_db().get_hist_total(
+        &h_date,
+        &h_state,
+        &h_counts
+    );
+
+    // Retrieving information on susceptibles
+    std::vector< double > S(this_const->get_ndays(), 0.0);
+    for (size_t i = 0; i < h_date.size(); ++i)
+    {
+        if (h_state[i] == "Susceptible")
+            S[h_date[i]] += h_counts[i];
+    }
+
+    // Computing the expected number of days in exposed
+    double days_exposed = this_const->par("Avg. Incubation days");
+
+    // The generation time in the SEIR model starts from 2, as agents 
+    // spend at least one day in the exposed state, and 1 day in the 
+    // infectious state before starting transmitting.
+    std::vector< double > gen_times(
+        this_const->get_ndays(), 1.0 + days_exposed
+        );
+        
+    double p_c = this_const->par("Contact rate")/this_const->size();
+    double p_i = this_const->par("Prob. Transmission");
+    double p_r = this_const->par("Prob. Recovery");
+
+    for (size_t i = 0u; i < this_const->get_ndays(); ++i)
+    {
+        gen_times[i] += gen_int_mean(
+            S[i],
+            p_c,
+            p_i,
+            p_r,
+            max_days,
+            max_contacts
+        );
+
+    }
+
+    return gen_times;
+
+}
+
 #endif
