@@ -16,7 +16,7 @@ private:
     int last_loc;     ///< Last location of the bar
     int cur_loc;      ///< Last location of the bar
     int i;            ///< Current iteration step
-    
+
 public:
     Progress() {};
     Progress(int n_, int width_);
@@ -24,16 +24,23 @@ public:
 
     void start();
     void next();
-    void end();
+    
 
 };
 
 inline Progress::Progress(int n_, int width_) {
 
 
+    if (n_ < 0)
+        throw std::invalid_argument("n must be greater or equal than 0.");
+
+    if (width_ <= 0)
+        throw std::invalid_argument("width must be greater than 0");
+
     width     = std::max(7, width_ - 7);
     n         = n_;
-    step_size = static_cast<epiworld_double>(width)/static_cast<epiworld_double>(n);
+    step_size = n == 0? width : static_cast<epiworld_double>(width)/
+        static_cast<epiworld_double>(n);
     last_loc  = 0;
     i         = 0;
 
@@ -58,27 +65,20 @@ inline void Progress::next() {
 
     cur_loc = std::floor((++i) * step_size);
 
-
     #ifndef EPI_DEBUG
     for (int j = 0; j < (cur_loc - last_loc); ++j)
-    {
+    { 
         printf_epiworld("|");
     }
-    #endif
 
-    if (i >= n)
-        end();
+    if (i == n)
+    {
+        printf_epiworld(" done.\n");
+    }
+    #endif
 
     last_loc = cur_loc;
 
 }
 
-inline void Progress::end() {
-
-    #ifndef EPI_DEBUG
-    printf_epiworld(" done.\n");
-    #endif
-
-}
-
 #endif
